# check.py | commands to check answers
# Copyright (C) 2019-2020  EraserBird, person_v1.32, hmmm

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

import discord
from discord.ext import commands

from sciolyid.data import database, get_wiki_url, logger, get_aliases
from sciolyid.functions import channel_setup, score_increment, spellcheck_list, user_setup


class Check(commands.Cog):
    def __init__(self, bot):
        self.bot = bot

    # Check command - argument is the guess
    @commands.command(help='- Checks your answer.', usage="guess", aliases=["guess", "c"])
    @commands.cooldown(1, 3.0, type=commands.BucketType.user)
    async def check(self, ctx, *, arg):
        logger.info("command: check")

        await channel_setup(ctx)
        await user_setup(ctx)

        currentItem = str(database.hget(f"channel:{str(ctx.channel.id)}", "item"))[2:-1]
        if currentItem == "":  # no image
            await ctx.send("You must ask for a image first!")
        else:  # if there is a image, it checks answer
            logger.info("currentItem: " + str(currentItem.lower().replace("-", " ")))
            logger.info("args: " + str(arg.lower().replace("-", " ")))

            if spellcheck_list(arg, get_aliases(currentItem.lower())) is True:
                logger.info("correct")

                database.hset(f"channel:{str(ctx.channel.id)}", "item", "")
                database.hset(f"channel:{str(ctx.channel.id)}", "answered", "1")

                
                database.zincrby("streak:global", 1, str(ctx.author.id))
                # check if streak is greater than max, if so, increases max
                if database.zscore("streak:global", str(ctx.author.id))> database.zscore("streak.max:global", str(ctx.author.id)):
                    database.zadd("streak.max:global", {str(ctx.author.id): database.zscore("streak:global", str(ctx.author.id))})

                await ctx.send("Correct! Good job!")
                url = get_wiki_url(currentItem)
                await ctx.send(url)
                score_increment(ctx, 1)

            else:
                logger.info("incorrect")
                
                database.zadd("streak:global", {str(ctx.author.id): 0})

                database.hset(f"channel:{str(ctx.channel.id)}", "item", "")
                database.hset(f"channel:{str(ctx.channel.id)}", "answered", "1")
                await ctx.send("Sorry, the image was actually " + currentItem.lower() + ".")
                url = get_wiki_url(currentItem)
                await ctx.send(url)


def setup(bot):
    bot.add_cog(Check(bot))



# skip.py | commands for skipping images
# Copyright (C) 2019-2020  EraserBird, person_v1.32, hmmm

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

from discord.ext import commands

from sciolyid.data import database, get_wiki_url, logger
from sciolyid.functions import channel_setup, user_setup


class Skip(commands.Cog):
    def __init__(self, bot):
        self.bot = bot

    # Skip command - no args
    @commands.command(help="- Skip the current image to get a new one", aliases=["sk", "s"])
    @commands.cooldown(1, 5.0, type=commands.BucketType.channel)
    async def skip(self, ctx):
        logger.info("command: skip")

        await channel_setup(ctx)
        await user_setup(ctx)

        currentItem = str(database.hget(f"channel:{str(ctx.channel.id)}", "item"))[2:-1]
        database.hset(f"channel:{str(ctx.channel.id)}", "item", "")
        database.hset(f"channel:{str(ctx.channel.id)}", "answered", "1")
        if currentItem != "":  # check if there is image
            url = get_wiki_url(currentItem)
            await ctx.send(f"Ok, skipping {currentItem.lower()}")
            await ctx.send(url)  # sends wiki page
            database.zadd("streak:global", {str(ctx.author.id): 0})  # end streak
        else:
            await ctx.send("You need to ask for an image first!")


def setup(bot):
    bot.add_cog(Skip(bot))



# media.py | commands for getting images
# Copyright (C) 2019-2020  EraserBird, person_v1.32, hmmm

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

import itertools
import random

from discord.ext import commands

import sciolyid.config as config
from sciolyid.core import send_image
from sciolyid.data import database, id_list, logger, groups
from sciolyid.functions import channel_setup, error_skip, user_setup, build_id_list

ARG_MESSAGE = f"**Recongnized arguments:** *{config.options['category_name']}*: `"+"{category}`"

IMAGE_MESSAGE = (
    f"*Here you go!* \n**Use `{config.options['prefixes'][0]}pic` again to get a new image of the same {config.options['id_type']}, " +
    f"or `{config.options['prefixes'][0]}skip` to get a new {config.options['id_type']}. Use `{config.options['prefixes'][0]}check [guess]` to check your answer. " +
    f"Use `{config.options['prefixes'][0]}hint` for a hint.**"
)

class Media(commands.Cog):
    def __init__(self, bot):
        self.bot = bot

    async def send_pic_(self, ctx, args):

        logger.info(f"{config.options['id_type']}: " + str(database.hget(f"channel:{str(ctx.channel.id)}", "item"))[2:-1])

        answered = int(database.hget(f"channel:{str(ctx.channel.id)}", "answered"))
        logger.info(f"answered: {answered}")
        # check to see if previous item was answered
        if answered:  # if yes, give a new item
            if config.options["id_groups"]:
                build = build_id_list(args)
                choices = build[0]
                await ctx.send(ARG_MESSAGE.format(category=build[1]))
            else:
                choices = id_list
            currentItem = random.choice(choices)
            prevB = str(database.hget(f"channel:{str(ctx.channel.id)}", "prevI"))[2:-1]
            while currentItem == prevB:
                currentItem = random.choice(choices)
            database.hset(f"channel:{str(ctx.channel.id)}", "prevI", str(currentItem))
            database.hset(f"channel:{str(ctx.channel.id)}", "item", str(currentItem))
            logger.info("currentItem: " + str(currentItem))
            database.hset(f"channel:{str(ctx.channel.id)}", "answered", "0")
            await send_image(ctx, currentItem, on_error=error_skip, message=IMAGE_MESSAGE)
        else:  # if no, give the same item
            await send_image(
                ctx,
                str(database.hget(f"channel:{str(ctx.channel.id)}", "item"))[2:-1],
                on_error=error_skip,
                message=IMAGE_MESSAGE
            )


    # Pic command - no args
    # help text
    @commands.command(help='- Sends a random image for you to ID', aliases=["p"])
    # 5 second cooldown
    @commands.cooldown(1, 5.0, type=commands.BucketType.channel)
    async def pic(self, ctx, *, args_str: str = ""):
        logger.info("command: pic")

        await channel_setup(ctx)
        await user_setup(ctx)

        if not config.options["id_groups"]:
            args_str = ""

        await self.send_pic_(ctx, args_str)


def setup(bot):
    bot.add_cog(Media(bot))



# race.py | commands for racing/competition
# Copyright (C) 2019-2020  EraserBird, person_v1.32, hmmm

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

import datetime
import time

import discord
from discord.ext import commands

from sciolyid.data import database, logger
from sciolyid.functions import channel_setup, user_setup


class Race(commands.Cog):
    def __init__(self, bot):
        self.bot = bot

    async def _get_options(self, ctx):
        bw, addon, state, media, limit = database.hmget(f"race.data:{str(ctx.channel.id)}",
                                                        ["bw", "addon", "state", "media", "limit"])
        options = str(
            f"**Age/Sex:** {str(addon)[2:-1] if addon else 'default'}\n" +
            f"**Black & White:** {bw==b'bw'}\n" +
            f"**Special bird list:** {str(state)[2:-1] if state else 'None'}\n" +
            f"**Media Type:** {str(media)[2:-1]}\n" +
            f"**Amount to Win:** {str(limit)[2:-1]}\n"
        )
        return options

    async def _send_stats(self, ctx, preamble):
        placings = 5
        database_key = f"race.scores:{str(ctx.channel.id)}"
        if database.zcard(database_key) == 0:
            logger.info(f"no users in {database_key}")
            await ctx.send("There are no users in the database.")
            return

        if placings > database.zcard(database_key):
            placings = database.zcard(database_key)

        leaderboard_list = database.zrevrangebyscore(
            database_key, "+inf", "-inf", 0, placings, True)
        embed = discord.Embed(
            type="rich", colour=discord.Color.blurple(), title=preamble)
        embed.set_author(name="Bird ID - An Ornithology Bot")
        leaderboard = ""

        for i, stats in enumerate(leaderboard_list):
            if ctx.guild is not None:
                user = ctx.guild.get_member(int(stats[0]))
            else:
                user = None

            if user is None:
                user = self.bot.get_user(int(stats[0]))
                if user is None:
                    user = "**Deleted**"
                else:
                    user = f"**{user.name}#{user.discriminator}**"
            else:
                user = f"**{user.name}#{user.discriminator}** ({str(user.mention)})"

            leaderboard += f"{str(i+1)}. {user} - {str(int(stats[1]))}\n"

        start = int(database.hget(f"race.data:{str(ctx.channel.id)}", "start"))
        elapsed = str(datetime.timedelta(seconds=round(time.time()) - start))

        embed.add_field(name="Options", value=await self._get_options(ctx), inline=False)
        embed.add_field(
            name="Stats", value=f"**Race Duration:** `{elapsed}`", inline=False)
        embed.add_field(name="Leaderboard", value=leaderboard, inline=False)

        if database.zscore(database_key, str(ctx.author.id)) is not None:
            placement = int(database.zrevrank(
                database_key, str(ctx.author.id))) + 1
            embed.add_field(
                name="You:", value=f"You are #{str(placement)}.", inline=False)
        else:
            embed.add_field(
                name="You:", value="You haven't answered any correctly.")

        await ctx.send(embed=embed)

    async def stop_race_(self, ctx):
        first = database.zrevrange(f"race.scores:{str(ctx.channel.id)}", 0, 0, True)[0]
        if ctx.guild is not None:
                user = ctx.guild.get_member(int(first[0]))
        else:
            user = None

        if user is None:
            user = self.bot.get_user(int(first[0]))
            if user is None:
                user = "Deleted"
            else:
                user = f"{user.name}#{user.discriminator}"
        else:
            user = f"{user.name}#{user.discriminator} ({str(user.mention)})"

        await ctx.send(f"**Congratulations, {user}!**\n" +
        f"You have won the race by correctly identifying `{str(int(first[1]))}` birds. " +
        "*Way to go!*")

        database.hset(f"race.data:{str(ctx.channel.id)}", "stop", round(time.time()))

        await self._send_stats(ctx, "**Race stopped.**")
        database.delete(f"race.data:{str(ctx.channel.id)}")
        database.delete(f"race.scores:{str(ctx.channel.id)}")

    @commands.group(brief="- Base race command",
                    help="- Base race command\n" +
                          "Races allow you to compete with others to see who can ID a bird first. " +
                          "Starting a race will keep all cooldowns the same, but automatically run " +
                          "'b!bird' (or 'b!song') after every check. You will still need to use 'b!check' " +
                          "to check your answer. Races are channel-specific, and anyone in that channel can play." +
                          "Races end when a player is the first to correctly ID a set amount of birds. (default 10)")
    async def race(self, ctx):
        if ctx.invoked_subcommand is None:
            await ctx.send('**Invalid subcommand passed.**\n*Valid Subcommands:* `start, view, stop`')

    @race.command(
        brief="- Starts race",
        help="""- Starts race.
        Arguments passed will become the default arguments to 'b!bird', but can be manually overwritten during use.
        Arguments can be passed in any order.
        However, having both females and juveniles are not supported.""",
        aliases=["st"],
        usage="[bw] [state] [female|juvenile] [amount to win (default 10)]"
    )
    @commands.cooldown(1, 3.0, type=commands.BucketType.channel)
    async def start(self, ctx, *, args_str: str = ""):
        logger.info("command: start race")

        await channel_setup(ctx)
        await user_setup(ctx)

        if ctx.guild is None:
            logger.info("dm context")
            await ctx.send("**Sorry, racing is not avaliable in DMs.**")
            return

        if not str(ctx.channel.name).startswith("racing"):
            logger.info("not race channel")
            await ctx.send("**Sorry, racing is not availiable in this channel.**\n" +
                           "*Set the channel name to start with `racing` to enable it.*")
            return

        if database.exists(f"race.data:{str(ctx.channel.id)}"):
            logger.info("already race")
            await ctx.send("**There is already a race in session.** *Change settings/view stats with `b!race view`*")
            return
        else:
            args = args_str.split(" ")
            logger.info(f"args: {args}")
            if "bw" in args:
                bw = "bw"
            else:
                bw = ""

            states_args = set(states.keys()).intersection(
                {arg.upper() for arg in args})
            if states_args:
                state = " ".join(states_args).strip()
            else:
                state = " ".join(check_state_role(ctx))

            female = "female" in args or "f" in args
            juvenile = "juvenile" in args or "j" in args
            if female and juvenile:
                await ctx.send("**Juvenile females are not yet supported.**\n*Please try again*")
                return
            elif female:
                addon = "female"
            elif juvenile:
                addon = "juvenile"
            else:
                addon = ""

            song = "song" in args or "s" in args
            image = "image" in args or "i" in args or "picture" in args or "p" in args
            if song and image:
                await ctx.send("**Songs and images are not yet supported.**\n*Please try again*")
                return
            elif song:
                media = "song"
            elif image:
                media = "image"
            else:
                media = "image"

            ints = []
            for n in args:
                try:
                    ints.append(int(n))
                except ValueError:
                    continue
            if len(ints) != 0:
                limit = int(ints[0])
            else:
                limit = 10

            if limit > 1000000:
                await ctx.send("**Sorry, the maximum amount to win is 1 million.**")
                limit = 1000000

            logger.info(
                f"adding bw: {bw}; addon: {addon}; state: {state}; media: {media}; limit: {limit}")

            database.hmset(
                f"race.data:{str(ctx.channel.id)}", {
                    "start": round(time.time()),
                    "stop": 0,
                    "limit": limit,
                    "bw": bw,
                    "state": state,
                    "addon": addon,
                    "media": media
                }
            )

            database.zadd(f"race.scores:{str(ctx.channel.id)}", {
                          str(ctx.author.id): 0})
            await ctx.send(f"**Race started with options:**\n{await self._get_options(ctx)}")

            if str(database.hget(f"race.data:{str(ctx.channel.id)}", "media"))[2:-1] == "image":
                logger.info("auto sending next bird image")
                addon, bw = map(str, database.hmget(
                    f"race.data:{str(ctx.channel.id)}", ["addon", "bw"]))
                birds = self.bot.get_cog("Birds")
                await birds.send_bird_(ctx, addon[2:-1], bw[2:-1])

            if str(database.hget(f"race.data:{str(ctx.channel.id)}", "media"))[2:-1] == "song":
                logger.info("auto sending next bird song")
                birds = self.bot.get_cog("Birds")
                await birds.send_song_(ctx)

    @race.command(
        brief="- Views race",
        help="- Views race.\nRaces allow you to compete with your friends to ID a certain bird first."
    )
    @commands.cooldown(1, 3.0, type=commands.BucketType.channel)
    async def view(self, ctx):
        logger.info("command: view race")

        await channel_setup(ctx)
        await user_setup(ctx)

        if database.exists(f"race.data:{str(ctx.channel.id)}"):
            await self._send_stats(ctx, f"**Race In Progress**")
        else:
            await ctx.send("**There is no race in session.** *You can start one with `b!race start`*")

    @race.command(help="- Stops race", aliases=["stp", "end"])
    @commands.cooldown(1, 3.0, type=commands.BucketType.channel)
    async def stop(self, ctx):
        logger.info("command: stop race")

        await channel_setup(ctx)
        await user_setup(ctx)

        if database.exists(f"race.data:{str(ctx.channel.id)}"):
            await self.stop_race_(ctx)
        else:
            await ctx.send("**There is no race in session.** *You can start one with `b!race start`*")


def setup(bot):
    bot.add_cog(Race(bot))



# sessions.py | commands for sessions
# Copyright (C) 2019-2020  EraserBird, person_v1.32, hmmm

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

import datetime
import time

import discord
from discord.ext import commands

from sciolyid.data import database, logger
from sciolyid.functions import channel_setup, user_setup


class Sessions(commands.Cog):
    def __init__(self, bot):
        self.bot = bot

    async def _get_options(self, ctx):
        bw, addon, state, order = database.hmget(f"session.data:{str(ctx.author.id)}",
                                                        ["bw", "addon", "state", "order"])
        options = str(
            f"**Age/Sex:** {str(addon)[2:-1] if addon else 'default'}\n" +
            f"**Black & White:** {bw==b'bw'}\n" +
            f"**State bird list:** {str(state)[2:-1] if state else 'None'}\n" +
            f"**Bird Order:** {str(order)[2:-1] if order else 'None'}\n"
        )
        return options

    async def _get_stats(self, ctx):
        start, correct, incorrect, total = map(
            int, database.hmget(f"session.data:{str(ctx.author.id)}", ["start", "correct", "incorrect", "total"]))
        elapsed = str(datetime.timedelta(seconds=round(time.time()) - start))
        try:
            accuracy = round(100 * (correct / (correct + incorrect)), 2)
        except ZeroDivisionError:
            accuracy = 0

        stats = str(
            f"**Duration:** `{elapsed}`\n" +
            f"**# Correct:** {correct}\n" +
            f"**# Incorrect:** {incorrect}\n" +
            f"**Total Birds:** {total}\n" +
            f"**Accuracy:** {accuracy}%\n"
        )
        return stats

    async def _send_stats(self, ctx, preamble):
        database_key = f"session.incorrect:{str(ctx.author.id)}"

        embed = discord.Embed(type="rich", colour=discord.Color.blurple(), title=preamble)
        embed.set_author(name="Bird ID - An Ornithology Bot")

        if database.zcard(database_key) != 0:
            leaderboard_list = database.zrevrangebyscore(
                database_key, "+inf", "-inf", 0, 5, True)
            leaderboard = ""

            for i, stats in enumerate(leaderboard_list):
                leaderboard += f"{str(i+1)}. **{str(stats[0])[2:-1]}** - {str(int(stats[1]))}\n"
        else:
            logger.info(f"no birds in {database_key}")
            leaderboard = "**There are no missed birds.**"

        embed.add_field(name="Options", value=await self._get_options(ctx), inline=False)
        embed.add_field(name="Stats", value=await self._get_stats(ctx), inline=False)
        embed.add_field(name=f"Top Missed Birds", value=leaderboard, inline=False)

        await ctx.send(embed=embed)

    @commands.group(brief="- Base session command",
                    help="- Base session command\n" +
                         "Sessions will record your activity for an amount of time and " +
                         "will give you stats on how your performance and " +
                         "also set global variables such as black and white, " +
                         "state specific bird lists, specific bird orders, or bird age/sex. ",
                    aliases=["ses", "sesh"])
    async def session(self, ctx):
        if ctx.invoked_subcommand is None:
            await ctx.send('**Invalid subcommand passed.**\n*Valid Subcommands:* `start, view, stop`')

    # starts session
    @session.command(
        brief="- Starts session",
        help="""- Starts session.
        Arguments passed will become the default arguments to 'b!bird', but can be manually overwritten during use. 
        These settings can be changed at any time with 'b!session edit', and arguments can be passed in any order. 
        However, having both females and juveniles are not supported.""",
        aliases=["st"],
        usage="[bw] [state] [female|juvenile] [order]"
    )
    @commands.cooldown(1, 3.0, type=commands.BucketType.user)
    async def start(self, ctx, *, args_str: str = ""):
        logger.info("command: start session")

        await channel_setup(ctx)
        await user_setup(ctx)

        if database.exists(f"session.data:{str(ctx.author.id)}"):
            logger.info("already session")
            await ctx.send("**There is already a session running.** *Change settings/view stats with `b!session edit`*")
            return
        else:
            args = args_str.split(" ")
            logger.info(f"args: {args}")
            if "bw" in args:
                bw = "bw"
            else:
                bw = ""
            states_args = set(states.keys()).intersection({arg.upper() for arg in args})
            if states_args:
                state = " ".join(states_args).strip()
            else:
                state = " ".join(check_state_role(ctx))
            order_args = set(orders["orders"]).intersection({arg.lower() for arg in args})
            if order_args:
                order = " ".join(order_args).strip()
            else:
                order = ""
            female = "female" in args or "f" in args
            juvenile = "juvenile" in args or "j" in args
            if female and juvenile:
                await ctx.send("**Juvenile females are not yet supported.**\n*Please try again*")
                return
            elif female:
                addon = "female"
            elif juvenile:
                addon = "juvenile"
            else:
                addon = ""
            logger.info(f"adding bw: {bw}; addon: {addon}; state: {state}")

            database.hmset(
                f"session.data:{str(ctx.author.id)}", {
                    "start": round(time.time()),
                    "stop": 0,
                    "correct": 0,
                    "incorrect": 0,
                    "total": 0,
                    "bw": bw,
                    "state": state,
                    "addon": addon,
                    "order": order
                }
            )
            await ctx.send(f"**Session started with options:**\n{await self._get_options(ctx)}")
    # views session
    @session.command(
        brief="- Views session",
        help="- Views session\nSessions will record your activity for an amount of time and " +
        "will give you stats on how your performance and also set global variables such as black and white, " +
        "state specific bird lists, specific bird orders, or bird age/sex. ",
        aliases=["view"],
        usage="[bw] [state] [female|juvenile]"
    )
    @commands.cooldown(1, 3.0, type=commands.BucketType.user)
    async def edit(self, ctx, *, args_str: str = ""):
        logger.info("command: view session")

        await channel_setup(ctx)
        await user_setup(ctx)

        if database.exists(f"session.data:{str(ctx.author.id)}"):
            args = args_str.split(" ")
            logger.info(f"args: {args}")
            if "bw" in args:
                if len(database.hget(f"session.data:{str(ctx.author.id)}", "bw")) == 0:
                    logger.info("adding bw")
                    database.hset(f"session.data:{str(ctx.author.id)}", "bw", "bw")
                else:
                    logger.info("removing bw")
                    database.hset(f"session.data:{str(ctx.author.id)}", "bw", "")
            states_args = set(states.keys()).intersection({arg.upper() for arg in args})
            if states_args:
                toggle_states = list(states_args)
                current_states = str(database.hget(f"session.data:{str(ctx.author.id)}", "state"))[2:-1].split(" ")
                add_states = []
                logger.info(f"toggle states: {toggle_states}")
                logger.info(f"current states: {current_states}")
                for state in set(toggle_states).symmetric_difference(set(current_states)):
                    add_states.append(state)
                logger.info(f"adding states: {add_states}")
                database.hset(f"session.data:{str(ctx.author.id)}", "state", " ".join(add_states).strip())
            order_args = set(orders["orders"]).intersection({arg.lower() for arg in args})
            if order_args:
                toggle_order = list(order_args)
                current_orders = str(database.hget(f"session.data:{str(ctx.author.id)}", "order"))[2:-1].split(" ")
                add_orders = []
                logger.info(f"toggle orders: {toggle_order}")
                logger.info(f"current orders: {current_orders}")
                for o in set(toggle_order).symmetric_difference(set(current_orders)):
                    add_orders.append(o)
                logger.info(f"adding orders: {add_orders}")
                database.hset(f"session.data:{str(ctx.author.id)}", "order", " ".join(add_orders).strip())
            female = "female" in args or "f" in args
            juvenile = "juvenile" in args or "j" in args
            if female and juvenile:
                await ctx.send("**Juvenile females are not yet supported.**\n*Please try again*")
                return
            elif female:
                addon = "female"
                if len(database.hget(f"session.data:{str(ctx.author.id)}", "addon")) == 0:
                    logger.info("adding female")
                    database.hset(f"session.data:{str(ctx.author.id)}", "addon", addon)
                else:
                    logger.info("removing female")
                    database.hset(f"session.data:{str(ctx.author.id)}", "addon", "")
            elif juvenile:
                addon = "juvenile"
                if len(database.hget(f"session.data:{str(ctx.author.id)}", "addon")) == 0:
                    logger.info("adding juvenile")
                    database.hset(f"session.data:{str(ctx.author.id)}", "addon", addon)
                else:
                    logger.info("removing juvenile")
                    database.hset(f"session.data:{str(ctx.author.id)}", "addon", "")
            await self._send_stats(ctx, f"**Session started previously.**\n")
        else:
            await ctx.send("**There is no session running.** *You can start one with `b!session start`*")

    # stops session
    @session.command(help="- Stops session", aliases=["stp"])
    @commands.cooldown(1, 3.0, type=commands.BucketType.user)
    async def stop(self, ctx):
        logger.info("command: stop session")

        await channel_setup(ctx)
        await user_setup(ctx)

        if database.exists(f"session.data:{str(ctx.author.id)}"):
            database.hset(f"session.data:{str(ctx.author.id)}", "stop", round(time.time()))

            await self._send_stats(ctx, "**Session stopped.**\n")
            database.delete(f"session.data:{str(ctx.author.id)}")
            database.delete(f"session.incorrect:{str(ctx.author.id)}")
        else:
            await ctx.send("**There is no session running.** *You can start one with `b!session start`*")

def setup(bot):
    bot.add_cog(Sessions(bot))



import setuptools


def readme():
    with open("README.md", "r") as f:
        return f.read()


setuptools.setup(
    name="sciolyid",
    version="0.0.3",
    description="Create ID Discord bots for SciOly studying.",
    long_description=readme(),
    long_description_content_type="text/markdown",
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)",
        "Programming Language :: Python :: 3.7",
    ],
    url="https://github.com/tctree333/SciOly-ID-Discord-Bots",
    author="Tomi Chen",
    author_email="tomichen33@gmail.com",
    packages=setuptools.find_packages(),
    install_requires=[
        "discord.py",
        "redis",
        "sentry-sdk",
        "Pillow",
        "wikipedia",
        "gitpython",
    ],
)
